@@ -16,8 +16,7 @@
 using namespace std;
 
 
-const array<const char*, sigfile::SChannel::n_channels>
-sigfile::SChannel::system1020_channels = {{  // counted 'em all!
+const char* sigfile::SChannel::system1020_channels[sigfile::SChannel::n_channels] = {  // counted 'em all!
 	"Nz",
 	"Fp1", "Fpz", "Fp2",
 	"AF7", "AF3", "AFz", "AF4", "AF8",
@@ -32,23 +31,23 @@ sigfile::SChannel::system1020_channels = {{  // counted 'em all!
 	// plus a few channels of other signal types
 	"Left", "Right",
 	"Chin",
-}};
+};
 
 
 
 
-const array<const char*, sigfile::SChannel::n_kemp_signal_types>
-sigfile::SChannel::kemp_signal_types = {{
+const char* sigfile::SChannel::kemp_signal_types[sigfile::SChannel::n_kemp_signal_types] = {
 	"EEG", "EOG", "EMG", "ECG", "ERG",
 	"NC",  "MEG", "MCG", "EP",
 	"Temp", "Resp", "SaO2",
 	"Light", "Sound", "Event", "Freq",
 	"(unknown)"
-}};
+};
 
 
 sigfile::SChannel::TType
-sigfile::SChannel::signal_type_of_channel( const string& signal)
+sigfile::SChannel::
+signal_type_of_channel( const string& signal)
 {
 	size_t h = 0;
 	for ( ; h <= last_eeg_no; ++h )
@@ -65,7 +64,9 @@ sigfile::SChannel::signal_type_of_channel( const string& signal)
 
 
 bool
-sigfile::SChannel::operator<( const SChannel& rv) const
+__attribute__ ((pure))
+sigfile::
+SChannel::operator<( const SChannel& rv) const
 {
 	size_t ai = 0, bi = 0;
 	while ( ai < n_channels && strcmp(    c_str(), system1020_channels[ai]) )
